Unit Displays;


(* Output device drivers for text and graphics *)


(******************************************************************************)
(*                                                                            *)
(*  This file is part of BO5, the user interface library for GNU Pascal.      *)
(*                                                                            *)
(*  BO5 will be distributed as free software (probably under the terms of     *)
(*  the GNU Library General Public License) as soon as it will have reached   *)
(*  some state of completion.  Up to that time, you are allowed to use BO5    *)
(*  for non-commercial purposes only.  You may distribute verbatim copies of  *)
(*  BO5 as a whole, but only without -- even a nominal -- fee.  You may not   *)
(*  distribute parts or changed versions of BO5.  I stress that there is      *)
(*  absolutely NO WARRANTY for BO5 -- any risk of using BO5 is with you!      *)
(*                                                                            *)
(*  Copyright (C) 1995  Peter Gerwinski, Duesseldorfer Str. 35                *)
(*                      (with TeX: D\"usseldorfer Str. 35)                    *)
(*                      45145 Essen, Germany                                  *)
(*                      e-mail: pege@mail.theo-phys.uni-essen.de              *)
(*                                                                            *)
(******************************************************************************)


(*$I CompOpts.icl *)


Interface


uses
  Tools, Objects;


(*** Simple screen color constants ***)


Const
  black     = 0;        darkgray     =  8;
  blue      = 1;        lightblue    =  9;
  green     = 2;        lightgreen   = 10;
  cyan      = 3;        lightcyan    = 11;
  red       = 4;        lightred     = 12;
  magenta   = 5;        lightmagenta = 13;
  brown     = 6;        yellow       = 14;
  lightgray = 7;        white        = 15;

  background = 16;


(*** Simple screen handling ***)


(* Some examples of what is meant by "simple screen handling":        *)
(*                                                                    *)
(*   WriteString ( 10, 5, 'Hello?', yellow + blue * background );     *)
(*                                                                    *)
(* writes a yellow 'Hello?' in a small blue box on the screen, such   *)
(* that 5 lines above it and 10 columns left of it remain free.       *)
(*                                                                    *)
(*   WriteChars ( 9, 4, 8, 3, ' ', yellow + blue * background );      *)
(*                                                                    *)
(* writes a box of width 8, height 3, upper left corner at (9,4) full *)
(* of yellow space characters on blue background (i.e. a blue box).   *)
(* When followed by the first example, it yields a yellow 'Hello?' in *)
(* a somewhat bigger blue box.                                        *)
(*                                                                    *)
(*   ClearScreen ( blue );                                            *)
(*                                                                    *)
(* sets the whole screen to blue.  When followed by the first         *)
(* example, you get a lonesome yellow 'Hello?' on a blue screen.      *)
(*                                                                    *)
(* In all these examples, the cursor is not moved.                    *)
(* Use "SetCursor ( x, y )" to put the cursor wherever you like -- it *)
(* is independend of the mentioned "Write-" procedures.               *)


Var
  ScreenWidth, ScreenHeight: Integer;
  CharWidth, CharHeight: Integer;
  CursorX, CursorY: Integer;


(* Coordinates are measured in "atom" units: An "atom" is a pixel in  *)
(* graphics mode or a character in text mode.  In text mode,          *)
(* CharWidth and CharHeight are set to 1.  If your application is     *)
(* planned to run in graphics as well as in text mode, you should al- *)
(* ways multiply text coordinates by CharWidth/CharHeight.            *)
(*                                                                    *)
(* Valid coordinates are 0..ScreenWidth - 1, 0..ScreenHeight - 1.     *)
(* Clipping does not yet work, but will do so in the next release.    *)

(* (Currently, there is no graphics mode implemented.  So just forget *)
(* about the above. :-)                                               *)

Procedure ClearScreen ( Attr: Byte );
Procedure SetCursor ( x, y: Integer );
Procedure HideCursor;
Procedure WriteString ( x, y: Integer; S: WrkString; Attr: Byte );
Procedure WriteChars ( x, y, w, h: Integer; C: Char; Attr: Byte );
Procedure WriteBuffer ( x, y, w: Integer; Var B: Void );


(*** Initialization and finalization (called automatically) ***)


Procedure InitDisplays;
Procedure FiniDisplays;


(*** Objects ***)


Type
  DisplayPtr = ^DisplayObj;


DisplayObj = object ( BaseObj )
  Size, CharSize, Cursor: Point;
  Constructor Init;
  Destructor Fini; virtual;
  Procedure Clear ( Attr: Byte ); virtual;
  Procedure SetCursor ( x, y: Integer ); virtual;
  Procedure HideCursor; virtual;
  Procedure WriteString ( x, y: Integer;
                          S: WrkString; Attr: Byte ); virtual;
  Procedure WriteChars ( x, y, w, h: Integer;
                         C: Char; Attr: Byte ); virtual;
  Procedure WriteBuffer ( x, y, w: Integer;
                          Var B: Void ); virtual;
end (* DisplayObj *);


Var
  Screen: DisplayPtr;


Implementation


(*$define DISPLAYS *)
  (*$I SysDep.icl *)
(*$undef DISPLAYS *)


(* The "simple screen handling" procedures call *)
(* the screen object's methods.  If there is no *)
(* screen object, they do some "dummy" action.  *)


Procedure ClearScreen ( Attr: Byte );

begin (* ClearScreen *)
  if Screen <> Nil then
    Screen^.Clear ( Attr );
end (* ClearScreen *);


Procedure SetCursor ( x, y: Integer );

begin (* SetCursor *)
  if Screen <> Nil then
    begin
      Screen^.SetCursor ( x, y );
      CursorX:= Screen^.Cursor.x;
      CursorY:= Screen^.Cursor.y;
    end (* if *)
  else
    begin
      CursorX:= max ( 0, min ( ScreenWidth - 1, x ) );
      CursorY:= max ( 0, min ( ScreenHeight - 1, y ) );
    end (* else *);
end (* SetCursor *);


Procedure HideCursor;

begin (* HideCursor *)
  if Screen <> Nil then
    Screen^.HideCursor;
end (* HideCursor *);


Procedure WriteString ( x, y: Integer; S: WrkString; Attr: Byte );

begin (* WriteString *)
  if Screen <> Nil then
    Screen^.WriteString ( x, y, S, Attr );
end (* WriteString *);


Procedure WriteChars ( x, y, w, h: Integer; C: Char; Attr: Byte );

begin (* WriteChars *)
  if Screen <> Nil then
    Screen^.WriteChars ( x, y, w, h, C, Attr );
end (* WriteChars *);


(*$W-*)  (* Suppress "dereferencing void* pointer" warning *)

Procedure WriteBuffer ( x, y, w: Integer; Var B: Void );

Var
  Buffer: Integer absolute B;

begin (* WriteBuffer *)
  if Screen <> Nil then
    Screen^.WriteBuffer ( x, y, w, Buffer );
end (* WriteBuffer *);

(*$W+*)


Procedure InitDisplays;

begin (* InitDisplays *)
  Screen:= New ( DisplayPtr, Init );
  ScreenWidth:= Screen^.Size.x;
  ScreenHeight:= Screen^.Size.y;
  CharWidth:= Screen^.CharSize.x;
  CharHeight:= Screen^.CharSize.y;
end (* InitDisplays *);


Procedure FiniDisplays;

begin (* FiniDisplays *)
  Dispose ( Screen, Fini );
  Screen:= Nil;
  (* Variables like ScreenWidth are not changed, *)
  (* so a valid "dummy" screen remains.          *)
end (* FiniDisplays *);


to begin 20 do
  InitDisplays;


to end do
  FiniDisplays;


end.
