Unit Tools;


(* General tools to make programmer's life easier *)


(******************************************************************************)
(*                                                                            *)
(*  This file is part of BO5, the user interface library for GNU Pascal.      *)
(*                                                                            *)
(*  BO5 will be distributed as free software (probably under the terms of     *)
(*  the GNU Library General Public License) as soon as it will have reached   *)
(*  some state of completion.  Up to that time, you are allowed to use BO5    *)
(*  for non-commercial purposes only.  You may distribute verbatim copies of  *)
(*  BO5 as a whole, but only without -- even a nominal -- fee.  You may not   *)
(*  distribute parts or changed versions of BO5.  I stress that there is      *)
(*  absolutely NO WARRANTY for BO5 -- any risk of using BO5 is with you!      *)
(*                                                                            *)
(*  Copyright (C) 1995  Peter Gerwinski, Duesseldorfer Str. 35                *)
(*                      (with TeX: D\"usseldorfer Str. 35)                    *)
(*                      45145 Essen, Germany                                  *)
(*                      e-mail: pege@mail.theo-phys.uni-essen.de              *)
(*                                                                            *)
(******************************************************************************)


(*$I CompOpts.icl *)


Interface


(*** pi ***)

Const
  pi = 3.141592653589793238462643383279502884197169;


(*** Data types ***)

Type
  ByteInt  = __byte__ Integer;        (* Signed Integer types with       *)
  ShortInt = __short__ Integer;       (* 8, 16, 64 Bits (Integer has 32) *)
  LongInt  = __longlong__ Integer;

  Byte      = __unsigned__ ByteInt;       (* Unsigned Integer types with *)
  ShortWord = __unsigned__ ShortInt;      (* 8, 16, 32, 64 Bits          *)
  Word      = __unsigned__ Integer;
  LongWord  = __unsigned__ LongInt;

  Pointer      = ^Void;                     (* General use Pointer types *)
  CharPtr      = ^Char;
  BytePtr      = ^Byte;
  ShortWordPtr = ^ShortWord;
  WordPtr      = ^Word;
  LongWordPtr  = ^LongWord;

  WrkString = String ( 255 );                  (* "Standard" string type *)


Point = record
  x, y: Integer;
end (* Point *);


(*** Tables for character conversion: UpCase, LoCase, CmpVal ***)

(* Currently, only the entries  0..127  make sense.  Ignore the others. :-) *)

Type
  CharTable = array [ Char ] of Char;

(*$I CharTables.icl *)


(*** Hex numbers ***)

(* Return a string of minimal length *)
(* containing x as a hex number:     *)
(* x = 31  yields  '1F'              *)

Function Hex ( x: Integer ): WrkString;


(*** Files ***)

(* Borland style file name assignment *)
Procedure Assign ( Var T: Text; Name: WrkString );


(*** Memory ***)

(* Fill memory with a given value *)
Procedure FillByte ( Var Dest: Void; Count: Integer; B: Byte );
Procedure FillShortWord ( Var Dest: Void; Count: Integer; W: ShortWord );
Procedure FillWord ( Var Dest: Void; Count: Integer; W: Word );
Procedure FillLongWord ( Var Dest: Void; Count: Integer; W: LongWord );

(* Borland/UCSD Pascal FillChar procedure *)
Procedure FillChar ( Var Dest: Void; Count: Integer; C: Char ); asmname 'Fillbyte';


(*** Command line parameters, system environment ***)

(* Borland compatible functions *)
Function ParamCount: Integer;
Function ParamStr ( i: Integer ): WrkString;
Function GetEnv ( Entry: WrkString ): WrkString;


(*** Random numbers ***)

(* UCSD style random integer *)
Function Random: Integer; asmname 'rand';

(* Initialize random number generator *)
Procedure InitRandom ( Seed: Integer ); asmname 'srand';

(* Return a random number 0 <= x < limit *)
Function RandInt ( limit: Integer ): Integer;
Function RandReal ( limit: Real ): Real;


(*** System environment information ***)

Const
  systemNone  = 0;

  (* EMX *)
  systemDOS     = 10;
  systemDPMI    = 11;  (* EMX or RSX, DPMI is available *)
  systemOS2     = 12;
  systemOS2pm   = 13;  (* not yet implemented *)

  (* UNIX *)
  systemTermCap = 20;
  systemX11     = 21;  (* not yet implemented *)
  systemLinux   = 22;


Var
  WhichSystem: Integer value systemNone;


(*** Errors ***)

(* Announce an error message and abort *)
Procedure FatalError ( Msg: WrkString );


(*** Initialization and finalization (called automatically) ***)

Procedure InitTools;
Procedure FiniTools;


(*** System dependend extra tools (for internal use only) *)

(*$define SYSTOOLS *)
  (*$I SysDep.icl *)
(*$undef SYSTOOLS *)


Implementation


Function Hex ( x: Integer ): WrkString;

Const
  HexTab: array [ 0..15 ] of Char =
    ( '0', '1', '2', '3', '4', '5', '6', '7',
      '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' );

Var
  i: Integer;
  S: WrkString;
 
begin (* Hex *)
  i:= 28;
  while ( i >= 0 ) and ( ( x shr i ) and $0F = 0 ) do
    dec ( i, 4 );
  if i < 0 then
    Hex:= '0'
  else
    begin
      S:= '';
      while i >= 0 do
        begin
          S:= S + HexTab [ ( x shr i ) and $0F ];
          dec ( i, 4 );
        end (* while *);
      Hex:= S;
    end (* else *);
end (* Hex *);


Procedure Assign ( Var T: Text; Name: WrkString );

Var
  B: BindingType;

begin (* Assign *)
  unbind ( T );
  B:= binding ( T );
  B.Name:= Name + chr ( 0 );
  bind ( T, B );
  B:= binding ( T );
end (* Assign *);


Procedure FillByte ( Var Dest: Void; Count: Integer; B: Byte );

Var
  p, q: BytePtr;

begin (* FillByte *)
  (*$W-*)  (* Warning "dereferencing `void *' pointer" is a minor bug in GPC *)
  p:= @Dest;
  (*$W+*)
  q:= BytePtr ( Integer ( p ) + Count );
  while Integer ( p ) < Integer ( q ) do
    begin
      p^:= B;
      inc ( Integer ( p ) );
    end (* while *);
end (* FillByte *);


Procedure FillShortWord ( Var Dest: Void; Count: Integer; W: ShortWord );

Var
  p, q: ShortWordPtr;

begin (* FillShortWord *)
  (*$W-*)
  p:= @Dest;
  (*$W+*)
  q:= ShortWordPtr ( Integer ( p ) + 2 * Count );
  while Integer ( p ) < Integer ( q ) do
    begin
      p^:= W;
      inc ( Integer ( p ), 2 );
    end (* while *);
end (* FillShortWord *);


Procedure FillWord ( Var Dest: Void; Count: Integer; W: Word );

Var
  p, q: WordPtr;

begin (* FillWord *)
  (*$W-*)
  p:= @Dest;
  (*$W+*)
  q:= WordPtr ( Integer ( p ) + 2 * Count );
  while Integer ( p ) < Integer ( q ) do
    begin
      p^:= W;
      inc ( Integer ( p ), 2 );
    end (* while *);
end (* FillWord *);


Procedure FillLongWord ( Var Dest: Void; Count: Integer; W: LongWord );

Var
  p, q: LongWordPtr;

begin (* FillLongWord *)
  (*$W-*)
  p:= @Dest;
  (*$W+*)
  q:= LongWordPtr ( Integer ( p ) + 4 * Count );
  while Integer ( p ) < Integer ( q ) do
    begin
      p^:= W;
      inc ( Integer ( p ), 4 );
    end (* while *);
end (* FillLongWord *);


(* Use the GPC run-time-system's ParamCount and ParamStr function *)

Function rtsParamCount: Integer;
AsmName '_p_paramcount';

Function rtsParamStr ( i: Integer; Var S: String ): Boolean;
AsmName '_p_paramstr';


Function ParamCount: Integer;  (* This is stupid. *)

begin (* ParamCount *)
  ParamCount:= rtsParamCount - 1;
end (* ParamCount *);


Function ParamStr ( i: Integer ): WrkString;

Var
  S: WrkString;

begin (* ParamStr *)
  if rtsParamStr ( i, S ) then
    ParamStr:= Trim ( S )
  else
    ParamStr:= '';
end (* ParamStr *);


Function CGetEnv ( Entry: CString ): CharPtr;
AsmName 'getenv';


Function GetEnv ( Entry: WrkString ): WrkString;

Var
  C: CharPtr;
  Contents: WrkString;

begin (* GetEnv *)
  C:= CGetEnv ( Entry );
  Contents:= '';
  if C <> Nil then
    while ( C^ <> chr ( 0 ) )
     and ( length ( Contents ) < Contents.Capacity ) do
      begin
        Contents:= Contents + C^;
        inc ( Word ( C ) );
      end (* while *);
  GetEnv:= Contents;
end (* GetEnv *);


Function RandInt ( limit: Integer ): Integer;

begin (* RandInt *)
  RandInt:= Random mod limit;
end (* RandInt *);


Function RandReal ( limit: Real ): Real;

begin (* RandReal *)
  RandReal:= limit * Random / $7FFFFFFF;
end (* RandReal *);


Procedure FatalError ( Msg: WrkString );

begin (* FatalError *)
  writeln;
  writeln ( 'bo5: ' + Msg );
  writeln;
  Halt ( -1 );
end (* FatalError *);


(*$define TOOLS *)
  (*$I SysDep.icl *)
(*$undef TOOLS *)


to begin 10 do
  InitTools;


to end do
  FiniTools;


end.
